#-calm
(ql:quickload :calm)
(in-package :calm)

(defun make-wasm (rebuild-wasm-p)
  (let* ((calm-version (slot-value (asdf:find-system 'calm) 'asdf:version))
         (web-tar-url (str:concat "https://github.com/VitoVan/calm/releases/download/" calm-version "/calm.tar"))
         (build-web-dir (merge-pathnames "build/web/" *calm-env-calm-home*))
         (web-tar-file (merge-pathnames "calm.tar" build-web-dir))
         (app-web-dir (merge-pathnames "web/" *calm-env-app-dir*))
         (build-dir (merge-pathnames "build-web/" *calm-env-app-dir*))
         (app-fonts-dir (merge-pathnames "fonts/" *calm-env-app-dir*))
         (app-assets-dir (merge-pathnames "assets/" *calm-env-app-dir*))
         (build-share-dir (merge-pathnames "share/" build-dir))
         (build-fonts-dir (merge-pathnames "fonts/" build-share-dir))
         (build-assets-dir (merge-pathnames "assets/" build-share-dir)))
    (ensure-directories-exist build-web-dir)
    (ensure-directories-exist app-web-dir)
    (if (string= rebuild-wasm-p "no")
        (progn
          (format t "rebuild-wasm-p was set as NO, using cache, or download it? let's see...~%")
          (if (probe-file web-tar-file)
              (format t "Using cached web tar file: ~A~%" web-tar-file)
              (progn
                (format t "Downloading web tar file from ~A ... ~%" web-tar-url)
                (u:exec (str:concat "curl -o " (uiop:native-namestring web-tar-file) " -L " web-tar-url))))
          (format t "extract files to ~A...~%" app-web-dir)
          (u:exec (str:concat "tar xvf " (uiop:native-namestring web-tar-file) " --strip-components=1 --directory=" (uiop:native-namestring app-web-dir))))
        (progn
          (format t "Building WASM ... ~%")
          ;; preparation and clean up
          (ensure-directories-exist build-share-dir)
          (u:touch-file (merge-pathnames "nothing" build-share-dir))
          (when (probe-file build-fonts-dir) (uiop:delete-directory-tree build-fonts-dir :validate t))
          (when (probe-file build-assets-dir) (uiop:delete-directory-tree build-assets-dir :validate t))
          ;; copy files
          (when (probe-file app-fonts-dir) (u:copy-dir app-fonts-dir build-fonts-dir))
          (when (probe-file app-assets-dir) (u:copy-dir app-assets-dir build-assets-dir))
          (ensure-directories-exist (merge-pathnames "src/" build-dir))
          (u:copy-dir (merge-pathnames "src/web" *calm-env-calm-home*) (merge-pathnames "src/web" build-dir))
          (format t "~%~%compiling... this will take one year, please be patient.~%~%~%")
          (u:exec
           (str:concat
            "docker run --rm --name pcwa -v "
            (uiop:native-namestring build-dir)
            ":/app -w=/app vitovan/pango-cairo-wasm "
            "emcc -Oz "
            "-sALLOW_MEMORY_GROWTH "
            "-sFULL_ES2 "
            "-s USE_SDL=2 "
            "-s USE_SDL_MIXER=2 "
            "-s USE_PTHREADS=0 "
            "--shell-file src/web/calm.html "
            "--pre-js src/web/pre.js "
            "--js-library src/web/bridge.js "
            "--preload-file ./share@/usr/share/ "
            "src/web/calm.c -o calm.html "
            (uiop:run-program "docker run --rm --name pcwa vitovan/pango-cairo-wasm pkg-config --libs --cflags glib-2.0, gobject-2.0, cairo, pixman-1, freetype2, fontconfig, cairo, expat, harfbuzz" :output '(:string :stripped t))
            "-s EXPORTED_RUNTIME_METHODS=ccall,cwrap,allocateUTF8,UTF8ToString "
            "-s EXPORTED_FUNCTIONS=_main,"
            "_cairo_create,"
            "_cairo_reference,"
            "_cairo_destroy,"
            "_cairo_status,"
            "_cairo_save,"
            "_cairo_restore,"
            "_cairo_get_target,"
            "_cairo_push_group,"
            "_cairo_push_group_with_content,"
            "_cairo_pop_group,"
            "_cairo_pop_group_to_source,"
            "_cairo_get_group_target,"
            "_cairo_set_source_rgb,"
            "_cairo_set_source_rgba,"
            "_cairo_set_source,"
            "_cairo_set_source_surface,"
            "_cairo_get_source,"
            "_cairo_set_antialias,"
            "_cairo_get_antialias,"
            "_cairo_set_dash,"
            "_cairo_get_dash_count,"
            "_cairo_get_dash,"
            "_cairo_set_fill_rule,"
            "_cairo_get_fill_rule,"
            "_cairo_set_line_cap,"
            "_cairo_get_line_cap,"
            "_cairo_set_line_join,"
            "_cairo_get_line_join,"
            "_cairo_set_line_width,"
            "_cairo_get_line_width,"
            "_cairo_set_miter_limit,"
            "_cairo_get_miter_limit,"
            "_cairo_set_operator,"
            "_cairo_get_operator,"
            "_cairo_set_tolerance,"
            "_cairo_get_tolerance,"
            "_cairo_clip,"
            "_cairo_clip_preserve,"
            "_cairo_clip_extents,"
            "_cairo_in_clip,"
            "_cairo_reset_clip,"
            "_cairo_rectangle_list_destroy,"
            "_cairo_copy_clip_rectangle_list,"
            "_cairo_fill,"
            "_cairo_fill_preserve,"
            "_cairo_fill_extents,"
            "_cairo_in_fill,"
            "_cairo_mask,"
            "_cairo_mask_surface,"
            "_cairo_paint,"
            "_cairo_paint_with_alpha,"
            "_cairo_stroke,"
            "_cairo_stroke_preserve,"
            "_cairo_stroke_extents,"
            "_cairo_in_stroke,"
            "_cairo_copy_page,"
            "_cairo_show_page,"
            "_cairo_get_reference_count,"
            "_cairo_set_user_data,"
            "_cairo_get_user_data,"
            "_cairo_copy_path,"
            "_cairo_copy_path_flat,"
            "_cairo_path_destroy,"
            "_cairo_append_path,"
            "_cairo_has_current_point,"
            "_cairo_get_current_point,"
            "_cairo_new_path,"
            "_cairo_new_sub_path,"
            "_cairo_close_path,"
            "_cairo_arc,"
            "_cairo_arc_negative,"
            "_cairo_curve_to,"
            "_cairo_line_to,"
            "_cairo_move_to,"
            "_cairo_rectangle,"
            "_cairo_glyph_path,"
            "_cairo_text_path,"
            "_cairo_rel_curve_to,"
            "_cairo_rel_line_to,"
            "_cairo_rel_move_to,"
            "_cairo_path_extents,"
            "_cairo_pattern_add_color_stop_rgb,"
            "_cairo_pattern_add_color_stop_rgba,"
            "_cairo_pattern_get_color_stop_count,"
            "_cairo_pattern_get_color_stop_rgba,"
            "_cairo_pattern_create_rgb,"
            "_cairo_pattern_create_rgba,"
            "_cairo_pattern_get_rgba,"
            "_cairo_pattern_create_for_surface,"
            "_cairo_pattern_get_surface,"
            "_cairo_pattern_create_linear,"
            "_cairo_pattern_get_linear_points,"
            "_cairo_pattern_create_radial,"
            "_cairo_pattern_get_radial_circles,"
            "_cairo_pattern_create_mesh,"
            "_cairo_mesh_pattern_begin_patch,"
            "_cairo_mesh_pattern_end_patch,"
            "_cairo_mesh_pattern_move_to,"
            "_cairo_mesh_pattern_line_to,"
            "_cairo_mesh_pattern_curve_to,"
            "_cairo_mesh_pattern_set_control_point,"
            "_cairo_mesh_pattern_set_corner_color_rgb,"
            "_cairo_mesh_pattern_set_corner_color_rgba,"
            "_cairo_mesh_pattern_get_patch_count,"
            "_cairo_mesh_pattern_get_path,"
            "_cairo_mesh_pattern_get_control_point,"
            "_cairo_mesh_pattern_get_corner_color_rgba,"
            "_cairo_pattern_reference,"
            "_cairo_pattern_destroy,"
            "_cairo_pattern_status,"
            "_cairo_pattern_set_extend,"
            "_cairo_pattern_get_extend,"
            "_cairo_pattern_set_filter,"
            "_cairo_pattern_get_filter,"
            "_cairo_pattern_set_matrix,"
            "_cairo_pattern_get_matrix,"
            "_cairo_pattern_get_type,"
            "_cairo_pattern_get_reference_count,"
            "_cairo_pattern_set_user_data,"
            "_cairo_pattern_get_user_data,"
            "_cairo_region_create,"
            "_cairo_region_create_rectangle,"
            "_cairo_region_create_rectangles,"
            "_cairo_region_copy,"
            "_cairo_region_reference,"
            "_cairo_region_destroy,"
            "_cairo_region_status,"
            "_cairo_region_get_extents,"
            "_cairo_region_num_rectangles,"
            "_cairo_region_get_rectangle,"
            "_cairo_region_is_empty,"
            "_cairo_region_contains_point,"
            "_cairo_region_contains_rectangle,"
            "_cairo_region_equal,"
            "_cairo_region_translate,"
            "_cairo_region_intersect,"
            "_cairo_region_intersect_rectangle,"
            "_cairo_region_subtract,"
            "_cairo_region_subtract_rectangle,"
            "_cairo_region_union,"
            "_cairo_region_union_rectangle,"
            "_cairo_region_xor,"
            "_cairo_region_xor_rectangle,"
            "_cairo_translate,"
            "_cairo_scale,"
            "_cairo_rotate,"
            "_cairo_transform,"
            "_cairo_set_matrix,"
            "_cairo_get_matrix,"
            "_cairo_identity_matrix,"
            "_cairo_user_to_device,"
            "_cairo_user_to_device_distance,"
            "_cairo_device_to_user,"
            "_cairo_device_to_user_distance,"
            "_cairo_select_font_face,"
            "_cairo_set_font_size,"
            "_cairo_set_font_matrix,"
            "_cairo_get_font_matrix,"
            "_cairo_set_font_options,"
            "_cairo_get_font_options,"
            "_cairo_set_font_face,"
            "_cairo_get_font_face,"
            "_cairo_set_scaled_font,"
            "_cairo_get_scaled_font,"
            "_cairo_show_text,"
            "_cairo_show_glyphs,"
            "_cairo_show_text_glyphs,"
            "_cairo_font_extents,"
            "_cairo_text_extents,"
            "_cairo_glyph_extents,"
            "_cairo_toy_font_face_create,"
            "_cairo_toy_font_face_get_family,"
            "_cairo_toy_font_face_get_slant,"
            "_cairo_toy_font_face_get_weight,"
            "_cairo_glyph_allocate,"
            "_cairo_glyph_free,"
            "_cairo_text_cluster_allocate,"
            "_cairo_text_cluster_free,"
            "_cairo_pattern_create_raster_source,"
            "_cairo_raster_source_pattern_set_callback_data,"
            "_cairo_raster_source_pattern_get_callback_data,"
            "_cairo_raster_source_pattern_set_acquire,"
            "_cairo_raster_source_pattern_get_acquire,"
            "_cairo_raster_source_pattern_set_snapshot,"
            "_cairo_raster_source_pattern_get_snapshot,"
            "_cairo_raster_source_pattern_set_copy,"
            "_cairo_raster_source_pattern_get_copy,"
            "_cairo_raster_source_pattern_set_finish,"
            "_cairo_raster_source_pattern_get_finish,"
            "_cairo_tag_begin,"
            "_cairo_tag_end,"
            "_cairo_image_surface_get_width,"
            "_cairo_image_surface_get_height,"
            "_cairo_image_surface_create_from_png,"
            "_cairo_surface_status,"
            "_cairo_status,"
            "_config,"
            "_Mix_OpenAudio,"
            "_Mix_CloseAudio,"
            "_Mix_LoadMUS,"
            "_Mix_LoadWAV_RW,"
            "_SDL_RWFromFile,"
            "_Mix_PlayMusic,"
            "_Mix_HaltMusic,"
            "_Mix_HaltChannel,"
            "_Mix_Volume,"
            "_Mix_VolumeMusic,"
            "_Mix_PlayChannelTimed,"
            "_Mix_Playing,"
            "_Mix_AllocateChannels,"
            "_SDL_GetTicks,"
            "_SDL_GetError,"
            "_free"))
          (u:calm-log-fancy "~%WASM file compiled.")))))

(make-wasm
 (u:get-from-env-or-ask 'rebuild-wasm-p "no"))
